import pytest
import respx
from httpx import Response

from keyrt import KeyRT
from tests.conftest import USER_RESPONSE_JSON


@pytest.mark.asyncio
@respx.mock
async def test_user(keyrt_client: KeyRT):
    respx.get("https://household.key.rt.ru/api/v3/app/users/current").mock(
        Response(status_code=200, content=USER_RESPONSE_JSON)
    )

    user = await keyrt_client.current_user()

    assert user.id == 123456
    assert user.login == "rtkid_1112223334445"



import pytest
import respx
from httpx import Response

from keyrt import KeyRT
from tests.conftest import DEVICES_RESPONSE_JSON


@pytest.mark.asyncio
@respx.mock
async def test_devices(keyrt_client: KeyRT):
    respx.get("https://household.key.rt.ru/api/v2/app/devices/intercom").mock(
        Response(status_code=200, content=DEVICES_RESPONSE_JSON)
    )

    devices = await keyrt_client.get_devices()

    assert len(devices) == 1


@pytest.mark.asyncio
@respx.mock
async def test_open_device(keyrt_client: KeyRT):
    respx.get("https://household.key.rt.ru/api/v2/app/devices/intercom").mock(
        Response(status_code=200, content=DEVICES_RESPONSE_JSON)
    )
    respx.post("https://household.key.rt.ru/api/v2/app/devices/131313/open").mock(
        Response(status_code=200)
    )

    devices = await keyrt_client.get_devices()

    assert await keyrt_client.open_device(devices[0].id)



from datetime import datetime
from typing import Any, List, Optional

from pydantic import BaseModel


class Capability(BaseModel):
    name: str
    setup: bool


class InterCode(BaseModel):
    id: int
    code: str
    start_date: datetime
    end_date: datetime | None
    inter_code_type: str


class Device(BaseModel):
    id: str
    device_type: str
    serial_number: str
    device_group: List[str]
    entrance: int
    utc_offset_minutes: int
    camera_id: str
    description: str
    is_favorite: bool
    is_active: bool
    name_by_company: str
    name_by_user: str | None
    accept_concierge_call: bool
    capabilities: List[Capability]
    inter_codes: List[InterCode]


class Devices(BaseModel):
    devices: List[Device]

    def __getitem__(self, item):
        # by device id
        for device in self.devices:
            if device.id == str(item):
                return device

        # by serial_number
        for device in self.devices:
            if device.serial_number == str(item):
                return device

        # by camera_id
        for device in self.devices:
            if device.camera_id == str(item):
                return device

        # by name_by_user
        for device in self.devices:
            if device.name_by_user == str(item):
                return device

        return self.devices[item]

    def __iter__(self):
        return iter(self.devices)

    def __len__(self):
        return len(self.devices)


class DevicesResponse(BaseModel):
    data: Devices



from typing import Any

from pydantic import BaseModel


class OauthKeyData(BaseModel):
    id_token: str
    token_type: str
    access_token: str
    refresh_token: str
    expires_in: int
    scope: str | list[str]


class OauthTokenData(BaseModel):
    key: OauthKeyData
    vc: Any


class OauthTokensResponse(BaseModel):
    request_id: str
    data: OauthTokenData
    error: Any



from typing import Any, List, Optional

from pydantic import BaseModel, validator, root_validator


class Company(BaseModel):
    id: int
    vc_id: int
    title: str
    utc_offset: int
    link_title: Any
    ios_deep_link: Any
    ios_app_store_link: Any
    android_package_id: Any
    huawei_appgallery_package_name: Any
    web_link: Any
    link_description: Any


class Account(BaseModel):
    number: int
    modified_at: str
    type: str


class UserSettings(BaseModel):
    redirect_to_mobile_application: bool
    devices_call_redirect_to_pstn: bool
    devices_call_redirect_to_sip: bool


class RoomPermissions(BaseModel):
    devices_rfids_intercom_access_behavior: bool
    devices_call_redirect_to_sip_behavior: bool
    devices_call_redirect_to_pstn_behavior: bool
    devices_call_redirect_to_mobile_app_behavior: bool
    devices_rfids_barrier_access_behavior: bool
    barriers_full_code_generate_behavior: bool
    devices_rfids_access_control_panel_access_behavior: bool
    fr_intercom_access_behavior: bool


class Flat(BaseModel):
    id: int
    company_id: int
    name: str
    area: int
    account: Account
    sips: List
    tfop_phones: List
    user_settings: UserSettings
    type: str
    room_permissions: RoomPermissions
    actions_permissions: List[str]
    is_subscription_enable: bool
    building_id: str
    entrance_id: str
    floor_id: str


class Floor(BaseModel):
    id: str
    name: str
    flats: List[Flat]
    building_id: str
    entrance_id: str

    @root_validator(pre=True)
    def entrance_flats(cls, values):
        for i in range(len(values["flats"])):
            values["flats"][i]["building_id"] = values["building_id"]
            values["flats"][i]["entrance_id"] = values["building_id"]
            values["flats"][i]["floor_id"] = values["id"]
        return values


class Entrance(BaseModel):
    id: str
    name: str
    floors: List[Floor]
    building_id: str

    @root_validator(pre=True)
    def entrance_floors(cls, values):
        for i in range(len(values["floors"])):
            values["floors"][i]["building_id"] = values["building_id"]
            values["floors"][i]["entrance_id"] = values["id"]
        return values


class Building(BaseModel):
    id: int
    mrf_id: int
    rf_id: int
    place: str
    street: str
    numbering: Any
    fias_id: Any
    orpon_id: int
    company_id: int
    utc_offset: int
    entrances: List[Entrance]

    @root_validator(pre=True)
    def entrance_enhancer(cls, values):
        for i in range(len(values["entrances"])):
            values["entrances"][i]["building_id"] = values["id"]
        return values


class User(BaseModel):
    id: int
    type: str
    login: str
    phone_number: str
    email: str
    status: str
    is_new: bool
    sso_id: str
    mos_sso_id: Any
    sso_user_id: int
    sso_linked: bool
    ory_id: str
    vc_id: int
    ustore_id: int
    mrf_id: int
    rf_id: int
    company: Company
    creator_user_id: int
    created_by: str
    phone_number_verified: bool
    email_verified: bool
    created_at: str
    archived_at: Any
    updated_at: str
    buildings: List[Building]

    @property
    def all_flats(self) -> list[Flat]:
        flats = []
        for build in self.buildings:
            for entrance in build.entrances:
                for floor in entrance.floors:
                    for flat in floor.flats:
                        flats.append(flat)

        return flats


class UserResponse(BaseModel):
    request_id: str
    data: User
    error: Any



from __future__ import annotations

from datetime import datetime
from typing import Any, Iterator, List, Optional

from pydantic import BaseModel, Field


class FlatItem(BaseModel):
    id: int
    number: str


class UserInfo(BaseModel):
    id: int
    email: str
    login: str
    phone_number: str
    status: str


class Company(BaseModel):
    id: int


class Device(BaseModel):
    id: int
    company: Company
    type: str
    title: str
    full_code: Any
    call_number: Any


class Status(BaseModel):
    device: Device
    status_changed_at: str
    status: str


class Code(BaseModel):
    id: int
    code: str
    type: str
    title: Any
    flat: Optional[FlatItem]
    company: Optional[Company]
    owner_type: str
    owner: Optional[UserInfo]
    creator_type: str
    creator: Optional[UserInfo]
    created_at: datetime
    begin_at: datetime
    expires_at: datetime
    statuses: List[Status]


class Codes(BaseModel):
    codes: List[Code] = Field(..., alias="items")
    total: int

    @property
    def temporary(self) -> Codes:
        codes = [code for code in self.codes if code.type == "temporary"]
        return Codes(items=codes, total=len(codes))

    @property
    def emergency(self) -> Codes:
        codes = [code for code in self.codes if code.type == "emergency"]
        return Codes(items=codes, total=len(codes))

    def __getitem__(self, item):
        # temporary by code id
        for code in self.temporary:
            if code.id == int(item):
                return code

        # temporary by device_id
        for code in self.temporary:
            if code.statuses[0].device.id == int(item):
                return code

        # temporary by name
        for code in self.temporary:
            if code.statuses[0].device.title == str(item):
                return code

        return self.codes[item]

    def __iter__(self) -> Iterator[Code]:
        return iter(self.codes)

    def __len__(self):
        return len(self.codes)


class CodesResponse(BaseModel):
    request_id: str
    data: Codes
    error: Any



import time
from base64 import b64encode, b64decode
from typing import Literal

from httpx import AsyncClient

from keyrt.models.cameras import Cameras, CamerasResponse
from keyrt.models.codes import Codes, CodesResponse
from keyrt.models.devices import DevicesResponse, Devices
from keyrt.models.tokens import OauthTokensResponse
from keyrt.models.user import User, UserResponse


class KeyRT:
    def __init__(self, access_token: str = None):
        self._access_token: str = access_token
        self._session: AsyncClient | None = None

    @property
    def session(self) -> AsyncClient:
        if self._session is None:
            self._session = AsyncClient(base_url="https://household.key.rt.ru/api")
        self._session.headers["Authorization"] = f"Bearer {self._access_token}"
        return self._session

    async def request(
        self,
        method: Literal["GET", "POST", "PUT", "DELETE"],
        url: str,
        params: dict | None = None,
        data: dict | None = None,
        json: dict | None = None,
        headers: dict | None = None,
        access_token: str | None = None,
    ):
        if access_token is not None:
            headers = (headers or {}) | {"Authorization": f"Bearer {access_token}"}
        return await self.session.request(
            method=method,
            url=url,
            headers=headers,
            params=params,
            json=json,
            data=data,
        )

    async def current_user(self, access_token: str = None) -> User:
        response = await self.request(
            method="GET",
            url="/v3/app/users/current",
            access_token=access_token,
        )

        return UserResponse(**response.json()).data

    async def get_devices(self, access_token: str = None) -> Devices:
        response = await self.request(
            method="GET",
            url="/v2/app/devices/intercom",
            access_token=access_token,
        )

        return DevicesResponse(**response.json()).data

    async def get_cameras(self, access_token: str = None) -> Cameras:
        response = await self.request(
            method="GET",
            url="https://vc.key.rt.ru/api/v1/cameras",
            params={
                "offset": 0,
                "limit": 1000,
            },
            access_token=access_token,
        )

        return CamerasResponse(**response.json()).data

    async def open_device(self, device_id: str, access_token: str = None) -> bool:
        response = await self.request(
            method="POST",
            url=f"/v2/app/devices/{device_id}/open",
            access_token=access_token,
        )

        return response.status_code == 200

    async def get_codes(self, access_token: str = None) -> Codes:
        response = await self.request(
            method="GET",
            url="/v3/app/devices/codes",
            access_token=access_token,
        )

        return CodesResponse(**response.json()).data

    async def generate_code(
        self, devices_ids: list[int] | int, flat_id: int, access_token: str = None
    ) -> bool:
        response = await self.request(
            method="POST",
            url="/v3/app/codes/generate",
            json={
                "devices_ids": [int(d) for d in devices_ids]
                if isinstance(devices_ids, list)
                else [int(devices_ids)],
                "flat_id": flat_id,
            },
            access_token=access_token,
        )

        return response.status_code == 202

    async def delete_code(self, flat_id: str, access_token: str = None) -> bool:
        response = await self.request(
            method="DELETE",
            url=f"/v2/app/flats/{flat_id}/intercode",
            access_token=access_token,
        )

        return response.status_code == 204

    def generate_oauth_link(self, callback_url: str = "https://tochka-api.pfel.cc"):
        url = (
            "https://b2c.passport.rt.ru/auth/realms/b2c/protocol/openid-connect/auth"
            "?redirect_uri=https://sso.key.rt.ru/api/v1/oauth2/b2c/callback"
            "&client_id=lk_dmh"
            "&response_type=code"
            f"&state={b64encode(f'{callback_url}?t={int(time.time())}'.encode()).decode()}"
        )
        return url

    async def oauth_to_token(
        self, code: str, state: str, auto_set_access_token: bool = True
    ) -> OauthTokensResponse:
        response = await self.request(
            method="POST",
            url="/v3/app/sso/oauth2/token",
            data={
                "code": code,
                "state": state,
                "timestamp": b64decode(state).decode().split("=")[-1],
                "grant_type": "authorization_code",
            },
        )

        if response.status_code == 200:
            response_data = OauthTokensResponse(**response.json())
            if auto_set_access_token:
                self._access_token = response_data.data.key.access_token

            return response_data

        return response.json()



import json

import pytest
import respx
from httpx import Request, Response

from keyrt import KeyRT
from tests.conftest import (
    CODES_RESPONSE_JSON,
    DEVICES_RESPONSE_JSON,
    USER_RESPONSE_JSON,
)


@pytest.mark.asyncio
@respx.mock
async def test_codes(keyrt_client: KeyRT):
    respx.get("https://household.key.rt.ru/api/v3/app/devices/codes").mock(
        Response(status_code=200, content=CODES_RESPONSE_JSON)
    )

    codes = await keyrt_client.get_codes()

    assert codes.total == 2
    assert len(codes.codes) == 2


@pytest.mark.asyncio
@respx.mock
async def test_generate_codes(keyrt_client: KeyRT):
    respx.get("https://household.key.rt.ru/api/v3/app/users/current").mock(
        Response(status_code=200, content=USER_RESPONSE_JSON)
    )
    respx.get("https://household.key.rt.ru/api/v2/app/devices/intercom").mock(
        Response(status_code=200, content=DEVICES_RESPONSE_JSON)
    )

    def check_codes_generate_request(request: Request):
        status_code = 400
        if (
            (data := json.loads(request.content.decode()))
            and data["devices_ids"] == [131313]
            and data["flat_id"] == 425783
        ):
            status_code = 202
        return Response(status_code)

    respx.post("https://household.key.rt.ru/api/v3/app/codes/generate").mock(
        side_effect=check_codes_generate_request,
    )

    user = await keyrt_client.current_user()
    devices = await keyrt_client.get_devices()

    assert await keyrt_client.generate_code(
        devices_ids=devices[0].id,
        flat_id=user.buildings[0].entrances[0].floors[0].flats[0].id,
    )


@pytest.mark.asyncio
@respx.mock
async def test_delete_codes(keyrt_client: KeyRT):
    respx.get("https://household.key.rt.ru/api/v3/app/users/current").mock(
        Response(status_code=200, content=USER_RESPONSE_JSON)
    )
    respx.delete("https://household.key.rt.ru/api/v2/app/flats/425783/intercode").mock(
        Response(status_code=204)
    )

    user = await keyrt_client.current_user()

    assert await keyrt_client.delete_code(
        flat_id=user.buildings[0].entrances[0].floors[0].flats[0].id,
    )



from datetime import datetime
from typing import Any, List, Optional, Iterator

from pydantic import BaseModel, Field


class Category(BaseModel):
    id: int
    title: str
    type: str


class Location(BaseModel):
    lat: Any
    lng: Any


class Status(BaseModel):
    id: int
    title: str
    type: str


class Camera(BaseModel):
    archive_length: Any
    category: Category
    created_at: str
    id: str
    ip: str
    location: Location
    mac: str
    model: str
    screenshot_precise_url_template: str
    screenshot_token: str
    screenshot_url_template: str
    serial_number: str
    status: Status
    streamer_token: str
    streamer_url: str
    title: str
    updated_at: datetime
    user_token: str
    utc_offset: int
    vendor: str


class Cameras(BaseModel):
    cameras: List[Camera] = Field(..., alias="items")
    total: int

    def __getitem__(self, item):
        # by camera id
        for camera in self.cameras:
            if camera.id == str(item):
                return camera

        # by title
        for camera in self.cameras:
            if camera.title == str(item):
                return camera

        return self.cameras[item]

    def __iter__(self) -> Iterator[Camera]:
        return iter(self.cameras)

    def __len__(self):
        return len(self.cameras)


class CamerasResponse(BaseModel):
    data: Cameras
    error: Any
    request_id: str



from io import open

from setuptools import setup


def read(filename):
    with open(filename, encoding="utf-8") as file:
        return file.read()


def requirements():
    with open("requirements.txt", "r") as req:
        return [r for r in req.read().split("\n") if r]


setup(
    name="keyrt",
    version="0.0.2a2",
    packages=["keyrt", "keyrt.models"],
    url="https://github.com/WhiteApfel/keyrt",
    license="Mozilla Public License 2.0",
    author="WhiteApfel",
    author_email="white@pfel.ru",
    description="KeyRT",
    install_requires=requirements(),
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
)



from .cameras import Camera, Cameras, CamerasResponse
from .codes import Code, Codes, CodesResponse
from .devices import Device, Devices, DevicesResponse
from .user import User, UserResponse
from .tokens import OauthTokensResponse
